
#pragma once
//#include "../myincludes.hh"
#include <cmath>

template<typename Number>
class Problem
{
  Number t;

  const Number TurningPointLeft  = 0.5 - 1/sqrt(12);
  const Number TurningPointRight = 0.5 + 1/sqrt(12);
public:
  typedef Number value_type;
  Number eps, delta;
  Number D;
  Number dt;
  Number uAst;
  Number u_regularization;
  Number mu;

  Number rho_0;

  Number rho_1;
  Number rho_2;

  Number M;
  Number M_reduced;
  Number fscaling;
  Number SurfaceTensionScaling;

  //Number rho (const Number& x) {return rho_0;}
  Number rho (const Number& x){return x*rho_2 + (1-x)*rho_1;}

  Number Sigma1,Sigma2,Sigma3,SigmaT;


  // Constructor takes parameter
  Problem (const Number& dt_, const Dune::ParameterTree& ptree)
           : t(0.0)
  {
    eps = ptree.get("problem.eps",(Number)0.1);
    delta = ptree.get("problem.delta",(Number)0.1);
    M = ptree.get("problem.Mobility",(Number)0.1);
    M_reduced = ptree.get("problem.MobilityReduced",(Number)0.1);
    u_regularization = ptree.get("problem.URegularization",(Number)0.01);

    D = ptree.get("physics.D",(Number)0.1);
    uAst = ptree.get("physics.uAst",(Number)0.1);
    rho_0 = ptree.get("physics.rho",(Number)1);
    mu = ptree.get("physics.mu",(Number)0.001);
    fscaling = ptree.get("phsyics.fscaling",(Number)1);
    SurfaceTensionScaling = ptree.get("physics.SurfaceTensionScaling",(Number)1);
    rho_1 = ptree.get("3p.rho1",(Number)1);
    rho_2 = ptree.get("3p.rho2",(Number)1);

    Sigma1 = ptree.get("3p.Sigma1",(Number)1);
    Sigma2 = ptree.get("3p.Sigma2",(Number)1);
    Sigma3 = ptree.get("3p.Sigma3",(Number)1);
    SigmaT = 1/(1/Sigma1 + 1/Sigma2 + 1/Sigma3);

    dt=dt_;
  }

  Number Mob (const Number& u) const
  {
    return M; //M_reduced + (M-M_reduced)*q(u);
  }

  //Double Well
  Number W (const Number& u) const
  {
    return u*u*(1-u)*(1-u);
  }
  // Double Well Derivative(/Secant)
  Number dW (const Number& x, const Number& y) const
  {
    return (x+y)*(1 +x*x +y*y) -2*(x*x +x*y +y*y);
  }

  // Double Well Derivative
  Number dW (const Number& x) const
  {
    return 2*x*(1-x)*(1-x) - 2*x*x*(1-x);
  }

  // Double Well Derivative of Convex Part
  Number dWConvex (const Number& x) const
  {
    if(x<TurningPointLeft)
      return dW(x)-dW(TurningPointLeft);
    if(x>TurningPointRight)
      return dW(x)-dW(TurningPointRight);
    return 0;
  }

  // Double Well Derivative of Convcarve Part
  Number dWConcarve (const Number& x) const
  {
    if(x<TurningPointLeft)
      return dW(TurningPointLeft);
    if(x>TurningPointRight)
      return dW(TurningPointRight);
    return dW(x);
  }

  Number dWdpf1 (const Number& pf1, const Number& pf2, const Number& pf3) const
  {
    return (Sigma1-SigmaT)*dW(pf1)-SigmaT*dW(pf2)-SigmaT*dW(pf3);
  }

  Number dWdpf2 (const Number& pf1, const Number& pf2, const Number& pf3) const
  {
    return (Sigma2-SigmaT)*dW(pf2)-SigmaT*dW(pf1)-SigmaT*dW(pf3);
  }

  Number dWdpf3 (const Number& pf1, const Number& pf2, const Number& pf3) const
  {
    return (Sigma3-SigmaT)*dW(pf3)-SigmaT*dW(pf1)-SigmaT*dW(pf2);
  }

  Number f (const Number& u) const
  {
    if(u<0)
      return fscaling*(-1.0+1000*u);
    else
      return fscaling*(u*u-1);
  }

  Number q (const Number& pf) const
  {
    if (pf<0 || pf>1)
      return 0;
    return pf*(1-pf);
  }

  Number q (const Number& pf1, const Number& pf2, const Number& pf3) const
  {
    if (pf1<0 || pf3<0)
      return 0;
    return pf1*pf3;
  }

  Number Diffusion(const Number& pf_c)
  {
    return D*fmax(1e-3,(pf_c-0.05)/0.95);
  }

  Number VelDissipation(const Number& pf_f)
  {
    return 10*(1-pf_f)*(1-pf_f);
  }

  //! Set time in instationary case
  void setTime (Number t_)
  {
    t = t_;
  }

};
